CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

        cols    = vga#cols
        rows    = vga#rows
        chrs    = cols * rows

        enter   = $0D
        bksp    = $C8
        left    = $C0
        right   = $C1
        up      = $C2
        down    = $C3
        tab     = $09
        f12     = $DB
        home    = $C4
        'inp     = $7000
        'attr    = $7C00
                                           
        _dopin = 0 
        _clkpin = 1 
        _dipin = 2 
        _cspin = 3 
        _cdpin = -1 ' -1 if unused. 
        _wppin = -1 ' -1 if unused. 
        _rtcclkpin = -1 ' -1 if unused. 
        _rtcdatpin = -1 ' -1 if unused. 
        _rtcbuslck = -1 ' -1 if unused.
        
        ' Use obj#errorName to get the error number constant. E.g. fat#disk_io_error.  
        #1, Disk_IO_Error, Clock_IO_Error, { 
          } File_System_Corrupted, File_System_Unsupported, { 
          } Card_Not_Detected, Card_Write_Protected, { 
          } Disk_May_Be_Full, Directory_Full, { 
          } Expected_An_Entry, Expected_A_Directory, { 
          } Entry_Not_Accessible, Entry_Not_Modifiable, { 
          } Entry_Not_Found, Entry_Already_Exist, { 
          } Directory_Link_Missing, Directory_Not_Empty, { 
          } Not_A_Directory, Not_A_File
        

VAR

  byte key
    
OBJ

  vga           : "vga_text"
  pst           : "parallax serial terminal"
  kb            : "keyboard"
  fat           : "SD-MMC_FATEngine.spin"
  str           : "string"

PUB main | i, errorNumber, errorString

  long [vga#crx] := 0
  long [vga#cry] := 0
  long [vga#crp] := 0

  vga.start(16)
  vga.clrScr
  
  kb.start(26, 27)
  pst.start(115200)
  fat.fatEngineStart( _dopin, _clkpin, _dipin, _cspin, _wppin, _cdpin, { 
                    } _rtcdatpin, _rtcclkpin, _rtcbuslck)

  errorString := \mount                                 ' Returns the address of the error string or null.
  pst.str(errorString) 
  errorNumber := fat.partitionError                     ' Returns the error number or zero. 
   
  if(errorNumber) ' Light a LED if an error occurs. 
    vga.str(string("SD Card error"), long[vga#crp])
    vga.char(enter, long[vga#crp])     
    'repeat ' Wait until reset or power down. 

  bytefill(@inp, 0, 256)
  bytefill(@attr, 0, 64)

  pst.str(fat.currentDir)
  
  repeat
    vga.str(@cl, long[vga#cry] * cols)
    
    bytefill(@inp, 0, 256)
    bytefill(@attr, 0, 64)

    i := 0
    repeat
      key := kb.getkey
      case key
        enter:
          vga.char(Enter, long[vga#crp])

        bksp:
          if (byte[vga#sbl] == 0 and long[vga#crx] <> 2) or (byte[vga#sbl] <> 0)
            vga.char(bksp, long[vga#crp])
            byte[@inp][i] := 0
            i--

        f12:
          vga.clrScr
          vga.str(@cl, 0)
          bytefill(inp, 0, 256) 

        $20..$7E:
          vga.char(key, long[vga#crp])
          byte[@inp][i] := key
          i++
          
        other:
          pst.hex(key, 3)
      
      'pst.str(@inp)      
    until key == enter
    
    if byte[@inp][0] <> 0 
      command
    
     

PUB command | cmdLength
 
  cmdLength := charPos(@inp, " ")

  'pst.dec(cmdLength)

  if cmdLength <> 0
    bytemove(@attr, @inp + cmdLength + 1, strLength(@inp) - cmdLength)
    bytefill(@inp + cmdLength, 0, strLength(@inp) - cmdLength - 1)
    
  pst.str(@inp) 
  pst.char(":")
  pst.str(@attr)
  pst.char(13)
       
  if str.stringCompareCS(@inp, string("ls")) == 0
    vga.char(tab, long[vga#crp])
    vga.str(string("List"), long[vga#crp])
    vga.char(enter, long[vga#crp])
    
  elseif str.stringCompareCS(@inp, string("cat")) == 0
    cat(@attr) 
    
  elseif str.stringCompareCS(@inp, string("mkdir")) == 0
    mkdir(@attr)
    
  elseif str.stringCompareCS(@inp, string("exit")) == 0
    vga.clrScr
    fat.unmountPartition
    fat.FATEngineStop
    vga.str(string("Waiting for power off..."), long [vga#crp])
    repeat
    
  elseif str.stringCompareCS(@inp, string("formatsd")) == 0
    vga.char(tab, long[vga#crp])
    vga.str(string("Formating..."), long[vga#crp])
    vga.char(enter, long[vga#crp])
    
    fat.formatPartition(0)
    
    vga.char(tab, long[vga#crp])
    vga.str(string("Format complete"), long[vga#crp])
    vga.char(enter, long[vga#crp])
    
  elseif str.stringCompareCS(@inp, string("cd")) == 0
    cd(@attr)
       
  else
    vga.char(tab, long[vga#crp])
    vga.str(string("Invalid command"), long[vga#crp])
    vga.char(enter, long[vga#crp])

PUB strLength(straddr)

  return charPos(straddr, 0)
  
PUB charPos(straddr, char) | length                     'Returns the number of bytes in a string up to a char, 0 if the char isn't present in the string

  length := 0

  repeat 
    length++
    straddr++
  until byte[straddr] == char or straddr == inp + 16

  if straddr == inp + 16
    length := 0    
  
  return length
   
PRI mount                                               ' Put the file system calls in a separate method to trap aborts.
 
  fat.mountPartition(0)                                 ' Mount the default partition 0.  Can be 0 - 3.

PRI mkdir(dir)

  fat.mountPartition(0)
  
  pst.str(fat.newDirectory(dir))
  pst.char(13)

  fat.unmountPartition

PRI cat(fileName)

  fat.mountPartition(0)

  if byte[fileName][0] == ">" and byte[fileName][1] == " "
  
    bytefill(fileName, 0, 2)
    'bytemove(fileName, fileName + 2, strLength(fileName))
    
    fat.openFile(fat.newFile(fileName + 2), "W")
    pst.str(string("file open, "))'
    vga.char(">", long[vga#crp])
        
    repeat
      pst.str(string("writing..."))
      key := kb.getkey
      case key
        enter:
          vga.char(Enter, long[vga#crp])
          fat.writeByte(13)

        bksp:
          if fat.fileTell > 0
            vga.char(bksp, long[vga#crp])
            fat.fileSeek(fat.fileTell - 1)
            fat.writeByte(0) 

        $20..$7E:
          vga.char(key, long[vga#crp])
          fat.writeByte(key)

        $D0:
          vga.char(enter, long[vga#crp])
          fat.closeFile
        
    until key == $D0 
    
  else
    vga.char(tab, long[vga#crp])
    vga.str(@catFormat, long[vga#crp])
    vga.char(enter, long[vga#crp])

  fat.unmountPartition

PRI cd(dir)

  fat.mountPartition(0)

  pst.str(string("cd"))
  pst.char(13)
  
  if fat.changeDirectory(dir) == fat#Not_A_Directory
    vga.char(tab, long[vga#crp])
    vga.str(string("No such directory"), long[vga#crp])
    vga.char(enter, long[vga#crp])
    pst.str(string("no directory"))
    pst.char(13)
  else
    pst.str(fat.changeDirectory(dir))
    
  fat.unmountPartition       
  
DAT

cl            byte      "$>", 0
catFormat     byte      "Use: cat > file.name", 0
inp           byte      0[256], 0
attr          byte      0[256], 0            